@@ -1,5 +1,7 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 
+#include <config.h>
+
 #define _ISOC99_SOURCE /* for roundf */
 #include <math.h>
 
@@ -11,10 +13,10 @@
 #include <clutter/x11/clutter-x11.h>
 #include <gdk/gdk.h> /* for gdk_rectangle_union() */
 
-#include "display.h"
-#include "errors.h"
+#include <meta/display.h>
+#include <meta/errors.h>
 #include "frame.h"
-#include "window.h"
+#include <meta/window.h>
 #include "xprops.h"
 
 #include "compositor-private.h"
@@ -22,6 +24,15 @@
 #include "meta-shaped-texture.h"
 #include "meta-window-actor-private.h"
 
+enum {
+  POSITION_CHANGED,
+  SIZE_CHANGED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = {0};
+
+
 struct _MetaWindowActorPrivate
 {
   XWindowAttributes attrs;
@@ -273,6 +284,21 @@ meta_window_actor_class_init (MetaWindowActorClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_SHADOW_CLASS,
                                    pspec);
+
+  signals[POSITION_CHANGED] =
+    g_signal_new ("position-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+  signals[SIZE_CHANGED] =
+    g_signal_new ("size-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -299,7 +325,6 @@ window_decorated_notify (MetaWindow *mw,
   MetaDisplay            *display  = meta_screen_get_display (screen);
   Display                *xdisplay = meta_display_get_xdisplay (display);
   Window                  new_xwindow;
-  MetaCompScreen         *info;
   XWindowAttributes       attrs;
 
   /*
@@ -315,8 +340,6 @@ window_decorated_notify (MetaWindow *mw,
 
   meta_window_actor_detach (self);
 
-  info = meta_screen_get_compositor_data (screen);
-
   /*
    * First of all, clean up any resources we are currently using and will
    * be replacing.
@@ -360,9 +383,6 @@ meta_window_actor_constructed (GObject *object)
   Window                  xwindow  = priv->xwindow;
   Display                *xdisplay = meta_display_get_xdisplay (display);
   XRenderPictFormat      *format;
-  MetaCompositor         *compositor;
-
-  compositor = meta_display_get_compositor (display);
 
 #ifdef HAVE_SHAPE
   /* Listen for ShapeNotify events on the window */
@@ -477,6 +497,12 @@ meta_window_actor_dispose (GObject *object)
 
   info->windows = g_list_remove (info->windows, (gconstpointer) self);
 
+  if (priv->window)
+    {
+      g_object_unref (priv->window);
+      priv->window = NULL;
+    }
+
   /*
    * Release the extra reference we took on the actor.
    */
@@ -509,7 +535,11 @@ meta_window_actor_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_META_WINDOW:
-      priv->window = g_value_get_object (value);
+      {
+        if (priv->window)
+          g_object_unref (priv->window);
+        priv->window = g_value_dup_object (value);
+      }
       break;
     case PROP_META_SCREEN:
       priv->screen = g_value_get_pointer (value);
@@ -881,6 +911,19 @@ meta_window_actor_get_texture (MetaWindowActor *self)
   return self->priv->actor;
 }
 
+/**
+ * meta_window_actor_is_destroyed:
+ *
+ * Gets whether the X window that the actor was displaying has been destroyed
+ *
+ * Return value: %TRUE when the window is destroyed, otherwise %FALSE
+ */
+gboolean
+meta_window_actor_is_destroyed (MetaWindowActor *self)
+{
+  return self->priv->disposed;
+}
+
 gboolean
 meta_window_actor_is_override_redirect (MetaWindowActor *self)
 {
@@ -1307,6 +1350,8 @@ meta_window_actor_sync_actor_position (MetaWindowActor *self)
                               window_rect.x, window_rect.y);
   clutter_actor_set_size (CLUTTER_ACTOR (self),
                           window_rect.width, window_rect.height);
+
+  g_signal_emit (self, signals[POSITION_CHANGED], 0);
 }
 
 void
@@ -1598,6 +1643,8 @@ meta_window_actor_update_bounding_region (MetaWindowActor *self,
    */
   if (!priv->shaped)
     meta_window_actor_invalidate_shadow (self);
+
+  g_signal_emit (self, signals[SIZE_CHANGED], 0);
 }
 
 static void
@@ -1762,7 +1809,6 @@ check_needs_pixmap (MetaWindowActor *self)
   MetaCompScreen      *info     = meta_screen_get_compositor_data (screen);
   MetaCompositor      *compositor;
   Window               xwindow  = priv->xwindow;
-  gboolean             full     = FALSE;
 
   if (!priv->needs_pixmap)
     return;
@@ -1823,9 +1869,11 @@ check_needs_pixmap (MetaWindowActor *self)
        * do it here.
        * See: http://bugzilla.clutter-project.org/show_bug.cgi?id=2236
        */
+#ifdef HAVE_GLX_TEXTURE_PIXMAP
       if (G_UNLIKELY (!clutter_glx_texture_pixmap_using_extension (
                                   CLUTTER_GLX_TEXTURE_PIXMAP (priv->actor))))
         g_warning ("NOTE: Not using GLX TFP!\n");
+#endif
 
       g_object_get (priv->actor,
                     "pixmap-width", &pxm_width,
@@ -1833,8 +1881,6 @@ check_needs_pixmap (MetaWindowActor *self)
                     NULL);
 
       meta_window_actor_update_bounding_region (self, pxm_width, pxm_height);
-
-      full = TRUE;
     }
 
   meta_error_trap_pop (display);
