@@ -35,10 +35,10 @@
 #define META_WINDOW_PRIVATE_H
 
 #include <config.h>
-#include "compositor.h"
-#include "window.h"
+#include <meta/compositor.h>
+#include <meta/window.h>
 #include "screen-private.h"
-#include "util.h"
+#include <meta/util.h>
 #include "stack.h"
 #include "iconcache.h"
 #include <X11/Xutil.h>
@@ -61,19 +61,13 @@ typedef enum {
 
 #define NUMBER_OF_QUEUES 3
 
-typedef enum {
-  META_TILE_NONE,
-  META_TILE_LEFT,
-  META_TILE_RIGHT,
-  META_TILE_MAXIMIZED
-} MetaTileMode;
-
 struct _MetaWindow
 {
   GObject parent_instance;
   
   MetaDisplay *display;
   MetaScreen *screen;
+  const MetaMonitorInfo *monitor;
   MetaWorkspace *workspace;
   Window xwindow;
   /* may be NULL! not all windows get decorated */
@@ -167,6 +161,11 @@ struct _MetaWindow
    */
   guint on_all_workspaces : 1;
 
+  /* This is true if the client requested sticky, and implies on_all_workspaces == TRUE,
+   * however on_all_workspaces can be set TRUE for other internal reasons too, such as
+   * being override_redirect or being on the non-primary monitor. */
+  guint on_all_workspaces_requested : 1;
+
   /* Minimize is the state controlled by the minimize button */
   guint minimized : 1;
   guint tab_unminimized : 1;
@@ -413,6 +412,10 @@ struct _MetaWindowClass
 #define META_WINDOW_TILED_SIDE_BY_SIDE(w)      ((w)->maximized_vertically && \
                                                 !(w)->maximized_horizontally && \
                                                  (w)->tile_mode != META_TILE_NONE)
+#define META_WINDOW_TILED_LEFT(w)     (META_WINDOW_TILED_SIDE_BY_SIDE(w) && \
+                                       (w)->tile_mode == META_TILE_LEFT)
+#define META_WINDOW_TILED_RIGHT(w)    (META_WINDOW_TILED_SIDE_BY_SIDE(w) && \
+                                       (w)->tile_mode == META_TILE_RIGHT)
 #define META_WINDOW_ALLOWS_MOVE(w)     ((w)->has_move_func && !(w)->fullscreen)
 #define META_WINDOW_ALLOWS_RESIZE_EXCEPT_HINTS(w)   ((w)->has_resize_func && !META_WINDOW_MAXIMIZED (w) && !META_WINDOW_TILED_SIDE_BY_SIDE(w) && !(w)->fullscreen && !(w)->shaded)
 #define META_WINDOW_ALLOWS_RESIZE(w)   (META_WINDOW_ALLOWS_RESIZE_EXCEPT_HINTS (w) &&                \
@@ -434,6 +437,7 @@ void        meta_window_unmanage           (MetaWindow  *window,
 void        meta_window_calc_showing       (MetaWindow  *window);
 void        meta_window_queue              (MetaWindow  *window,
                                             guint queuebits);
+void        meta_window_tile               (MetaWindow        *window);
 void        meta_window_maximize_internal  (MetaWindow        *window,
                                             MetaMaximizeFlags  directions,
                                             MetaRectangle     *saved_rect);
@@ -627,5 +631,7 @@ void meta_window_update_icon_now (MetaWindow *window);
 
 void meta_window_update_role (MetaWindow *window);
 void meta_window_update_net_wm_type (MetaWindow *window);
+void meta_window_update_monitor (MetaWindow *window);
+void meta_window_update_on_all_workspaces (MetaWindow *window);
 
 #endif
